package cam.codegen;
import java.io.*;
import java.util.Vector;
import java.util.Date;
import java.text.DateFormat;

//
//#####################################################################
//                  CLASS CreateClassInterface
//
//  A class for creating an interface of public methods that the input
//  class implements.
//
//  Input  <-- className
//  Output --> classNameIf (class interface)
//
//  The creation of the interface must be re-done anytime the set of public
//  methods of the  input class changes.
//
public class CreateClassInterface
{
public void setClass(String C) throws ClassNotFoundException
{
    className = C;
    classInfo = new ClassInterrogator();
    classInfo.setClass(className);
}

public String getLocalClassName()
{
    return classInfo.getClassName();
}

public void setSourceDirectory(String sourceD)
{
   sourceFileDirectory = sourceD; 
}
//
// This code is a bit convoluted, but the general idea is to 
// create the interface to a class that implements that interface ...
// e.g. there is a circular dependence that must be avoided.
//
// Step #1 : Copy .java file to a temporary class file, renaming the
//           class name with dummy names

// Step #2 : Compile the temporary class file, and create interface code
//           for it.

// Step #3 : Change the names in the temporary interface code back to
//           the names of the original class.
//
//
// In order for this procedure to work, the temporary files must
// be written in the directory that contains the source file ... thus
// programs calling this class need to invoke the setSourceDirectory(...)
//

public String [] createInterfaceCode() throws FileNotFoundException
{ 
    String packageName = classInfo.getPackageName();
    String localClass  = classInfo.getClassName();
    String tmpName = "ZZZTemp";
    //
    // first read in the source class
    //
    File inFileS;
    BufferedReader fileInputS = null;
    inFileS          = new File(sourceFileDirectory + localClass +".java");
    fileInputS       = new BufferedReader(new FileReader(inFileS));
    //
    // Create temp file
    //
    File outFile;
    PrintWriter fileOutput = null;

    outFile           = new File(sourceFileDirectory + tmpName +".java");
    try{fileOutput    = new PrintWriter(new FileOutputStream(outFile),true);}
    catch(IOException e){System.out.println(e.getMessage());}
    //
    // replace all occurances references to the class with a dumy instance
    // with a dumy instance
    //
    int firstIndex = -1; 
    String Astring; String Bstring;
    
    String IfName = new String(localClass + "If");
    String line = null;
    try
    {
    line = fileInputS.readLine();
    
    while(line != null)
    {

      firstIndex = line.indexOf(IfName);
      while(firstIndex >= 0)
      {
      Astring    = line.substring(0,firstIndex);
      Bstring    = line.substring(firstIndex + IfName.length(),line.length());
      line       = new String(Astring + "cam.codegen.NullIf" + Bstring);
      firstIndex = line.indexOf(IfName);
      }
      
      firstIndex = line.indexOf(localClass);
      while(firstIndex >= 0)
      {
      Astring    = line.substring(0,firstIndex);
      Bstring    = line.substring(firstIndex + localClass.length(),line.length());
      line       = new String(Astring + tmpName + Bstring);
      firstIndex = line.indexOf(localClass);
      }
      fileOutput.println(line);
      line = fileInputS.readLine();
    }
    }catch(Exception e){};
    //
    // Invoke the compiler on the temp file
    //
    sun.tools.javac.Main compiler = new sun.tools.javac.Main(System.out, "javac");
    String [] cArgs1 = {sourceFileDirectory + tmpName + ".java"};
    compiler.compile(cArgs1);
    //
    // delete the temp source file
    //
    fileOutput.close();
    outFile.delete();
    
    try
    {
    if(packageName != null)
    {setClass(packageName + "." + tmpName);}
    else
    {setClass(tmpName);}
    }catch(Exception e){}
    
    String [] R = createCode();
    
    //
    // remove temp class file
    //
    outFile = new File(sourceFileDirectory + tmpName + ".class");
    outFile.delete();
    //
    // replace temp names with correct names 
    //

    int i;
    for(i = 0; i < R.length; i++)
    {
    line = R[i];
    firstIndex = line.indexOf(tmpName);
    while(firstIndex >= 0)
    {
      Astring    = line.substring(0,firstIndex);
      Bstring    = line.substring(firstIndex + tmpName.length(),line.length());
      line       = new String(Astring + localClass + Bstring);
      firstIndex = line.indexOf(tmpName);
    }
     R[i] = new String(line);
    }

    return R;
}
//
// By interrogating the class loaded with the setClass command, this
// routine creates the interface code for it.
//
public String [] createCode() 
{

    int i; int j;
    StringBuffer sb;
    Vector cLines = new Vector();
    //
    // create package name
    //
    String packageName = classInfo.getPackageName();
    if(packageName != null)
    {cLines.addElement("package " + classInfo.getPackageName() + ";");}

    cLines.addElement("//");
    cLines.addElement("//#################################################################");
    cLines.addElement("//      Class " + classInfo.getClassName() + " Interface  ");
    cLines.addElement("//#################################################################");
    cLines.addElement("//");
    Date JobDate;
    JobDate = new Date();
    String dateOut =
    DateFormat.getDateTimeInstance(DateFormat.SHORT,DateFormat.SHORT).format(JobDate);
    cLines.addElement("// Construction Date : " + dateOut);
    cLines.addElement("// Auto-generated by CreateClassInterface (Chris Anderson (C) UCLA 1998)");
    cLines.addElement("//");
    //
    //  class declaration : name
    //
    cLines.addElement("public interface " + classInfo.getClassName() + "If");
    //
    //  body of class
    //
    cLines.addElement("{");
    cLines.addElement(" ");
    // capture names
    //
    String [] methodNames = classInfo.getMethodNames();
    String [] returnNames = classInfo.getReturnNames();
    String [] parameterNames;
    String [] argumentNames;
    String [] exceptionNames;
    int       exceptionCount;
    int       parameterCount;
    int       parameterCountp1;

    //
    // loop over all public methods, and create code for them
    //
    for(i = 0; i < classInfo.getMethodCount(); i++)
    {
       // compose method name :

       sb = new StringBuffer();

       // pubilc + return type  + method

       sb.append("  public " + returnNames[i] + " " + methodNames[i] + "(");
       parameterNames = classInfo.getParameterNames(i);
       argumentNames  = classInfo.getArgumentNames(i);
       exceptionNames = classInfo.getExceptionNames(i);

       // ( parameters )

       parameterCount = parameterNames.length;
       if(parameterNames.length >= 1)
       {sb.append(parameterNames[0] + " " + argumentNames[0]);}
       for(j = 1; j < parameterNames.length; j++)
       {sb.append("," + parameterNames[j] + " " + argumentNames[j]);}
       sb.append(")");

       // exceptions

       exceptionCount = exceptionNames.length;
       if(exceptionCount >= 1)
       {sb.append(" throws " + exceptionNames[0]);}
        for(j = 1; j < exceptionCount; j++)
       {sb.append(", " + exceptionNames[j]);}


       cLines.addElement(sb.toString()+ ";");
    }
    cLines.addElement(" ");
    cLines.addElement("}");
    String [] returnString = new String[cLines.size()];
    for(i = 0; i < returnString.length; i++)
    {
    returnString[i] = (String)cLines.elementAt(i);
    }
    return returnString;
}
    String className;
    ClassInterrogator classInfo;
    String sourceFileDirectory;


static public void main(String args[])
{
    int i; int j;
    String inputClass;

    if(args.length == 0) return;
    inputClass = args[0];

    CreateClassInterface codeCreator = new CreateClassInterface();
    try{codeCreator.setClass(inputClass);}
    catch(ClassNotFoundException e){System.out.println(e.getMessage());}
    
    String localClassName = codeCreator.getLocalClassName();

    System.out.println("Building Interface To Class : " + localClassName + "CI");

    String [] interfaceCode = null;
    try
    {
        
    interfaceCode  = codeCreator.createInterfaceCode();
    
    //
    //Screen Output
    //

    //for(i = 0; i < interfaceCode.length; i++)
    //{
    //    System.out.println(interfaceCode[i]);
    //}


    File outFileI;
    PrintWriter fileOutputI = null;

    outFileI          = new File(localClassName + "If.java");
    try{fileOutputI   = new PrintWriter(new FileOutputStream(outFileI),true);}
    catch(IOException e){};

    for(i = 0; i < interfaceCode.length; i++)
    {
     fileOutputI.println(interfaceCode[i]);
    }
    System.out.println("Finished Interface To Class : " + localClassName + "If.java");

    //
    // Invoke the compiler
    //
    sun.tools.javac.Main compiler = new sun.tools.javac.Main(System.out, "javac");
    String [] cArgs1 = {localClassName + "If.java"};
    compiler.compile(cArgs1);
    
    
    }
    catch(FileNotFoundException e)
    {System.out.println("Error Source " + e.getMessage() +
    " File Not Found");}
    
    
    System.out.println("Done : Hit return to continue ");
    BufferedReader D =
    new BufferedReader(new InputStreamReader(System.in));
    try{D.readLine();}catch(Exception e){}
}

}
